package com.ddignani;

 
import google.zxing.integration.android.IntentIntegrator;
import google.zxing.integration.android.IntentResult;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.ActivityInfo;
import android.database.Cursor;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.provider.Settings;
import android.telephony.TelephonyManager;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.ddignani.network.Facade;

public class KeepInQActivity extends ListActivity {
	
	static final int RESERVATION_ACTIVITY = 1;
	static final int GPS_ACTIVITY = 2;
	static final boolean DEFAULT_AUTOUPDATEACTIVE = true;
	static final long DEFAULT_AUTOUPDATEINTERVAL = 300000;	// 5 minuti
	static final long AUTOUPDATEINTERVAL_ON_LOW_BAT = DEFAULT_AUTOUPDATEINTERVAL * 2;
	static final long AUTOUPDATEINTERVAL_ON_OKAY_BAT = DEFAULT_AUTOUPDATEINTERVAL; 
	private static final float DISTANCE_MAX = 20;	// Distanza massima dal qr code
	
	Facade mFacade;
	ReservationsDbAdapter dbAdapter;
	ReservationCursorAdapter reservations;
	
	// Valori delle preferenze
	private boolean isAutoUpdateSelected = DEFAULT_AUTOUPDATEACTIVE;
	private long autoUpdateInterval = DEFAULT_AUTOUPDATEINTERVAL;
	private long autoUpdateIntervalOnBattery = DEFAULT_AUTOUPDATEINTERVAL; 
	
	private Handler mHandler;
	private AutoUpdateTask autoUpdateTask;
	private boolean isAutoUpdateRunning = true;
	private SharedPreferences preferences;
	
	private LocationManager locationManager;
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);	// Progressbar nel titolo inizialmente non visibile
        setContentView(R.layout.main);	// La view vuota la prende dal layout xml
        
        //crea l'adapter per usare il database
        dbAdapter = new ReservationsDbAdapter(this);
        dbAdapter.removeAllReservation();
           
        // Registro il menu contestuale
        registerForContextMenu(getListView());	
        
        // Recupera il codice IMEI del dispositivo dell'utente: utile per la Facade
        TelephonyManager tm = (TelephonyManager)getSystemService(Context.TELEPHONY_SERVICE); 
		mFacade = new Facade(this, dbAdapter, tm.getDeviceId()); 
        
        // Pulsante per la nuova Reservation
        Button button = (Button) findViewById(R.id.new_reserv);
        button.setOnClickListener(new Button.OnClickListener() {
            public void onClick(View v) {
                newReservation();
            }
        });
    
        // Recupera le preferenze
    	preferences = PreferenceManager.getDefaultSharedPreferences(this);
    	PreferenceManager.setDefaultValues(this, R.xml.preferences, false);
    	
        // Strutture dati per l'Autoupdater
        if(mHandler == null)
        	mHandler = new Handler();
        if(autoUpdateTask == null)
        	autoUpdateTask = new AutoUpdateTask();
        
        
    }
   
    /**
     * Riempie la listView di Reservations, se ce ne sono nel database
     */
    public void fillData() {
    	
    	Cursor cursor = dbAdapter.fetchAllReservations();
    	startManagingCursor(cursor);
    	
    	reservations = new ReservationCursorAdapter(this, cursor, false);
    	setListAdapter(reservations);
    	
    	if(isAutoUpdateRunning){ //Sta andando
    		if(!isAutoUpdateSelected){ // Non dovrebbe andare
    			stopAutoUpdater();
    		}else{
    			// Dovrebbe andare ma se il db  vuoto o ci sono solo reservation gi Ready allora dico di no
    			if(dbAdapter.isEmpty() || dbAdapter.isFullOfReady()){
    				stopAutoUpdater();
    			}
    		}	
    	}else{	//Non sta andando
    		if(isAutoUpdateSelected){ //Dovrebbe andare
    			if(!dbAdapter.isEmpty() && !dbAdapter.isFullOfReady()){ // occorre che vada? solo se non vuoto e se non sono tutti Ready
    				startAutoUpdater();
    			}
    		}
    	}
    }
    
    /**
     * Avvia il servizio di autoUpdate
     */
    private void startAutoUpdater(){
    	mHandler.postDelayed(autoUpdateTask, autoUpdateInterval);
		isAutoUpdateRunning = true;
    }
    
    /**
     * Sospende il servizio di autoUpdate
     */
    private void stopAutoUpdater(){
		mHandler.removeCallbacks(autoUpdateTask);
		isAutoUpdateRunning = false;
    }
    


	public void onStart(){
    	super.onStart();
    	// Lettura delle preferences
    	isAutoUpdateSelected = preferences.getBoolean("autoUpdateCheckBox", DEFAULT_AUTOUPDATEACTIVE);
    	if(isAutoUpdateSelected){
    		autoUpdateInterval = Long.parseLong(preferences.getString("updates_interval", "-1"));
    		if(autoUpdateInterval == -1){
    			autoUpdateInterval = autoUpdateIntervalOnBattery;
    		}
    	}	 
	}

	@Override
	public void onResume(){
		super.onResume();
    	fillData();
    }
	
	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		super.onListItemClick(l, v, position, id);
		Intent i = new Intent(this, ReservationActivity.class);
		i.putExtra(ReservationsDbAdapter.KEY_ROWID, id);
		startActivityForResult(i, RESERVATION_ACTIVITY);
	}

	// Creazione menu o ActionBar se API > 10
    public boolean onCreateOptionsMenu(Menu menu){
    	getMenuInflater().inflate(R.menu.options_menu, menu);
    	return true;
    }

    // Aggiornamento menu: si pu rinunciare o aggiornare tutte le reservation solo se ce ne  almeno una...
    public boolean onPrepareOptionsMenu(Menu menu){	
    	//se non ci sono reservation allora non mostro le opzioni di gruppo
    	menu.setGroupVisible(R.id.group_all, !dbAdapter.isEmpty());
    	return super.onPrepareOptionsMenu(menu);
    }

    // Azioni menu  
    public boolean onOptionsItemSelected(MenuItem item){		
		switch(item.getItemId()){   
			case R.id.about:
				showAbout();
				break;
			case R.id.decline_all:
				declineAll();
				break;
			case R.id.preferences:
				showPreferences();
				break;
			case R.id.refresh_all:
				updateAll();
				break; 
		}
		return super.onOptionsItemSelected(item);
	}


	private void updateAll() {
		mFacade.updateAllReservations();
			
	}

	
	//Per le prove
//	private void newReservation(){
////		String serverInfo = "Agenzia delle Entrate|10.0.2.2|8080|unbreakable";  // con emulatore
//			String serverInfo = "Agenzia delle Entrate|192.168.0.169|8888|unbreakable"; //con dispositivo
//		mFacade.newReservation(serverInfo);
//	}

	//Acquisizione con ZXing
    private void newReservation() {
    	locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
    	final boolean gpsEnabled = locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER);
    	if(!gpsEnabled){
    		// Mostro Dialog che chieda all'utente di abilitare il gps
    		final AlertDialog alert = new AlertDialog.Builder(this).create();

    		alert.setTitle(R.string.gps_not_available);
    		alert.setMessage(getResources().getText(R.string.gps_not_available_dialog));
    		//Crea il pulsante S
    		alert.setButton(DialogInterface.BUTTON_POSITIVE, getResources().getText(R.string.yes), new DialogInterface.OnClickListener() {
    			@Override
    			public void onClick(DialogInterface dialog, int which) {
    				Intent settingsIntent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
    				startActivityForResult(settingsIntent,GPS_ACTIVITY);
    			}
    		});
    		//Crea il pulsante NO
    		alert.setButton(DialogInterface.BUTTON_NEGATIVE, getResources().getText(R.string.no), new DialogInterface.OnClickListener() {		
    			@Override
    			public void onClick(DialogInterface dialog, int which) {
    				dialog.cancel();
    			}
    		});
    		alert.show();
    	}
    	else{
    		IntentIntegrator integrator = new IntentIntegrator(this);
    		integrator.initiateScan(IntentIntegrator.QR_CODE_TYPES);
    	}
    }

	
	public void onActivityResult(int requestCode, int resultCode, Intent intent) {
		// L'utente  ritornato dalla ReservationActivity
		if(requestCode == RESERVATION_ACTIVITY){
			//L'utente ha effettivamente richiesto un azione
			if(resultCode == RESULT_OK){				
				Bundle extras = intent.getExtras();
				long id = extras.getLong(ReservationsDbAdapter.KEY_ROWID);
				
				switch (intent.getExtras().getInt(ReservationActivity.ACTION)) {
				case ReservationActivity.ACTION_DECLINE:
					mFacade.decline(id);
					break;
				case ReservationActivity.ACTION_REFRESH:
					updateReservation(id);
					break;
				default:
					break;
				}
				
				fillData();
			}
			
		}
		else if(requestCode == GPS_ACTIVITY){
			locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
	  	    final boolean nowEnabled = locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER);
	  	    if(!nowEnabled){
	  	    	// evidentemente l'utente  tornato senza abilitare il gps...
	  	    	Toast.makeText(KeepInQActivity.this, "GPS is still disabled: cant' proceed", Toast.LENGTH_LONG).show();
	  	    }else{
	  	    	IntentIntegrator integrator = new IntentIntegrator(this);
	  	    	integrator.initiateScan(IntentIntegrator.QR_CODE_TYPES);
	  	    }
		}
		//L'utente ha scansionato il codice
		else{ 
			IntentResult scanResult = IntentIntegrator.parseActivityResult(requestCode, resultCode, intent);
			if (scanResult != null) {
				String contents = scanResult.getContents();
				
				//per prove senza GPS
//				if (contents != null) {
//					String serverInfo = intent.getStringExtra("SCAN_RESULT");
//					mFacade.newReservation(serverInfo);
//				}
				
				if (contents != null) {
					// Divido tra informazioni del server e posizione fisica
					String[] decoded = intent.getStringExtra("SCAN_RESULT").split("@");
					if(decoded.length != 2){
						Toast.makeText(getBaseContext(), "Testo QR mal formattato: manca la '@' tra dati e posizione", Toast.LENGTH_LONG).show();
						return;
					}
					String serverInfo = decoded[0];
					// Divido tra latitutdine e longitudine
					String[] coord = decoded[1].split(",");
					if(coord.length != 2){
						Toast.makeText(getBaseContext(), "Testo QR mal formattato: manca la ',' tra lat e lon", Toast.LENGTH_LONG).show();
						return;
					}
					if(isCloseToQR(coord[0], coord[1]))
						mFacade.newReservation(serverInfo);
					else{
						Toast.makeText(getBaseContext(), "Too far from QR-code", Toast.LENGTH_LONG).show();
					}
				} else {
					fillData();
				}

				
			}
		}
	}

	
	/**
	 * Calcola se il device  vicino al QR
	 * @param lat Latitudine del QR
	 * @param lon Longitudine del QR
	 * @return true se distanza minore di DISTANCE_MAX metri
	 */
    private boolean isCloseToQR(String lat, String lon) {
		
    	Criteria criteria = new Criteria();
		criteria.setAccuracy(Criteria.ACCURACY_FINE);
		criteria.setAltitudeRequired(false);
		criteria.setBearingRequired(false);
		criteria.setCostAllowed(false);

		String provider = locationManager.getBestProvider(criteria, false);
    	Location devicePosition = locationManager.getLastKnownLocation(provider);
    	
    	float[] results = new float[1];
		Location.distanceBetween(devicePosition.getLatitude(), devicePosition.getLongitude(), Location.convert(lat),Location.convert(lon), results);
//		Log.i("gps", results[0] + " metri");
		return results[0] < DISTANCE_MAX;
	}



	// Chiede se cancellare tutte le reservation
	private void declineAll() {
		final AlertDialog alert = new AlertDialog.Builder(this).create();
		alert.setTitle(R.string.decline_all);
		alert.setMessage(getResources().getText(R.string.decline_all_dialog));
		
		//Crea il pulsante S
		alert.setButton(DialogInterface.BUTTON_POSITIVE, getResources().getText(R.string.yes), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {				
				mFacade.declineAll();
			}
		});
		//Crea il pulsante NO
		alert.setButton(DialogInterface.BUTTON_NEGATIVE, getResources().getText(R.string.no), new DialogInterface.OnClickListener() {		
			@Override
			public void onClick(DialogInterface dialog, int which) {
				alert.dismiss();
			}
		});
		//Mostra il dialog
		alert.show();
	}
	
	//Mostra la schermata delle preferenze
	private void showPreferences() {
	          startActivity(new Intent(this, PreferencesActivity.class));		
	}
	
	// Mostra l'about
	private void showAbout() {
		//set up dialog
        final Dialog dialog = new Dialog(this);
        
        dialog.setContentView(R.layout.about_dialog);
        dialog.setCancelable(true);
        
        //set up button
        Button button = (Button) dialog.findViewById(R.id.about_button);
        button.setOnClickListener(new OnClickListener() {
        @Override
            public void onClick(View v) {
                dialog.dismiss();
            }
        });
        dialog.show();
		
	}

	//Creazione menu contestuale
	public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
		super.onCreateContextMenu(menu, v, menuInfo);

		AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo) menuInfo;
        menu.setHeaderTitle(((TextView) info.targetView.findViewById(R.id.row_name)).getText());
		getMenuInflater().inflate(R.menu.context_menu, menu);
		
	}
	
	//Azioni menu contestuale
	public boolean onContextItemSelected(MenuItem item){
		//Recupera la posizione del elemento da eliminare
		AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
		switch(item.getItemId()) {
        	case R.id.decline_reserv:
        		declineReservation(info.id);        		
        		break;
        		
        	case R.id.refresh_reserv:
        		updateReservation(info.id);
        		break;
        	
        	case R.id.confirm_reserv:
        		confirmReservation(info.id);
        		break;	
        		
		}
		return super.onContextItemSelected(item);
	}

	/**
	 *  Chiede se cancellare la Reservation selezionata
	 * @param idRes Id della Reservation da cancellare
	 */
	public void declineReservation(long idRes) {
		final long id = idRes;// per usarla dentro l'onClick pi sotto.
		final AlertDialog alert = new AlertDialog.Builder(this).create();
		
		alert.setTitle(R.string.decline_reserv);
		alert.setMessage(getResources().getText(R.string.decline_reserv_dialog));
		//Crea il pulsante S
		alert.setButton(DialogInterface.BUTTON_POSITIVE, getResources().getText(R.string.yes), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				mFacade.decline(id);
			}
		});
		//Crea il pulsante NO
		alert.setButton(DialogInterface.BUTTON_NEGATIVE, getResources().getText(R.string.no), new DialogInterface.OnClickListener() {		
			@Override
			public void onClick(DialogInterface dialog, int which) {
				dialog.cancel();
			}
		});
		alert.show();
		
	}
	
	/**
	 * Aggiorna la reservation selezionata
	 * @param id Id della Reservation da aggiornare
	 */
	private void updateReservation(long id) {
			mFacade.updateReservation(id);
	}
	
	/**
	 * Conferma la reservation selezionata
	 * @param id
	 */
	private void confirmReservation(long id) {
		newReservation();
		
	}


	/**
	 * Classe per l'aggiornamento automatico delle reservation
	 */
	
	public class AutoUpdateTask implements Runnable {
		public AutoUpdateTask(){}
		
		@Override
		public void run() {
			mFacade.autoUpdateReservations();		
			// riavvio l'updater
			mHandler.postDelayed(this, autoUpdateInterval);
		}

	}
	
	public class BatteryStatusReceiver extends BroadcastReceiver{

		@Override
		public void onReceive(Context context, Intent intent) {
			if(intent.getAction() == "ACTION_BATTERY_LOW"){
				autoUpdateIntervalOnBattery = AUTOUPDATEINTERVAL_ON_LOW_BAT;
			}
			if(intent.getAction() == "ACTION_BATTERY_OKAY"){
				autoUpdateIntervalOnBattery = AUTOUPDATEINTERVAL_ON_OKAY_BAT;
			}	
		}
	}
	
	public long getNextUpdateInterval(){
		return autoUpdateInterval;
	}
	
	
}