package com.ddignani;

import android.text.format.Time;

public class Reservation{
	
	public static final int PENDING = 0;
	public static final int READY = 1;
	public static final int CONFIRMED = 2;
	public static final int ERROR = -1;
	
	private long id;				//ID della reservation
	private String name;			//Nome della Reservation
	private String address;			//Indirizzo del server
	private int port;				//Porta di ascolto del server
	private String idClient;		//ID assegnato dal server
	private String password;		//password per accedere al server
	private long waiting = 0;		//tempo da aspettare
	private long lastUpdate;		//orario dell'ultimo aggiornamento
	private int status;				//Stato della prenotazione
	private String msgFromServer;	//stato della prenotazione

	

	public Reservation (String name, String address, int port, String password){
		this.name = name;
		this.address = address;
		this.port = port;
		this.setPassword(password);
		this.setStatus(PENDING);
	}
	
	public String toString(){
		StringBuilder summary = new StringBuilder();
		summary.append(this.name + "|");
		summary.append(this.address + "|");
		summary.append(this.port + "|");
		summary.append(this.password + "|");
		summary.append(this.waiting + "|");
		summary.append(this.lastUpdate + "|");
		summary.append(this.status + "|");
		summary.append(this.msgFromServer);
		
		return summary.toString();
	}
	
	public boolean isEqual(Reservation res){
		return (res.getAddress() == this.address && res.getIdClient() == this.idClient);
	}
	
	
	public void setAddress(String address) {
		this.address = address;
	}
	
	public String getAddress() {
		return this.address;
	}

	public void setPort(int port) {
		this.port = port;
	}
	
	public int getPort() {
		return this.port;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	public String getName() {
		return this.name;
	}

	public void setId(long id){
		this.id = id;
	}
	
	public long getId() {
		return id;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getMsgFromServer() {
		return msgFromServer;
	}

	public void setMsgFromServer(String msg) {
		this.msgFromServer = msg;
	}

	public String getIdClient() {
		return idClient;
	}

	public void setIdClient(String idClient) {
		this.idClient = idClient;
	}
	
	/**
	 * Restituisce l'ultimo aggiornamento
	 * @return ultimo aggiornamento
	 */
	public long	getLastUpdate() {
		return lastUpdate;
	}
	/**
	 * Aggiorna il campo dell'ultimo update con l'orario corrente
	 */
	public void setLastUpdate(){
		lastUpdate = System.currentTimeMillis();
	}
	/**
	 * Imposta l'istante di ultimo aggiornamento (utile quando si recuperano i dati dal database)
	 * @param time
	 */
	public void setLastUpdate(long time){
		lastUpdate = time;
	}

	/**
	 * Imposta il tempo di attesa e aggiorna il campo LasUpdate
	 * @param time
	 */
	public void setWaiting(long time) {
		this.waiting = time;
	}
	/**
	 * Restituisce il tempo di attesa
	 * @return Tempo di attesa in millisecondi
	 */
	public long getWaiting(){
		return waiting;
	}
	
	public String getReadableServiceTime(){
		Time time = new Time();
		time.set(lastUpdate + waiting);
		return time.format("%H:%M");
	}

	public int getStatus() {
		return status;
	}

	public void setStatus(int status) {
		this.status = status;
	}
}
