package com.ddignani;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

public class ReservationsDbAdapter {
	
	private static final String DATABASE_NAME = "KeepInQ.db";
	public static final String DATABASE_TABLE = "mainTable";
	private static final int DATABASE_VERSION = 1;
	
	//Colonne nella tabella (usata per fare fetch)
	public static final String KEY_ROWID = "_id";
	public static final String KEY_NAME = "name";
	public static final String KEY_ADDRESS = "address";
	public static final String KEY_PORT = "port";
	public static final String KEY_PASSWORD = "password";
	public static final String KEY_WAITING = "waiting";
	public static final String KEY_LASTUPDATE = "lastUpdate";
	public static final String KEY_STATUS = "status";
	public static final String KEY_MSG = "msgFromServer";
	
	
	/**
	 * Array di String per indicare le colonne della tabella nel DB
	 */
	private static final String[] ALL_COLUMNS = new String[] {KEY_ROWID, KEY_NAME, KEY_ADDRESS, KEY_PORT, KEY_PASSWORD, KEY_WAITING, KEY_LASTUPDATE, KEY_STATUS, KEY_MSG};
	private static final String[] LISTVIEW_COLUMNS = new String[] {KEY_ROWID, KEY_NAME, KEY_WAITING, KEY_LASTUPDATE, KEY_STATUS, KEY_MSG};
	
	private final Context context;
	private SQLiteDatabase db;
	private DBHelper dbHelper;
	
	private boolean isOpen = false;
	private int rowCount;	//indica quante righe ci sono nel db
	private int rowReady;	//indica quante di queste righe sono gi Ready (-> non occorre aggiornarle)
	
	/**
	 * Query usata per creare il DB
	 */
	private static final String DATABASE_CREATE = "create table if not exists " + DATABASE_TABLE + "( " 
			+ KEY_ROWID + " integer primary key autoincrement, " 
			+ KEY_NAME + " text not null, "
			+ KEY_ADDRESS + " text not null, "
			+ KEY_PORT + " integer, "
			+ KEY_PASSWORD + " text, "
			+ KEY_WAITING + " long, "
			+ KEY_LASTUPDATE + " long, "
			+ KEY_STATUS + " int, "
			+ KEY_MSG + " text"
			+ ");";
	

	public ReservationsDbAdapter(Context context){
		this.context = context;
	}
	
	/**
	 * Apre una connessione al DB usando un DBHElper 
	 * (nello stesso file di ReservationsDbAdapter)
	 * @return DBAdapter del database
	 * @throws SQLException
	 */
	public ReservationsDbAdapter open() throws SQLException{
		dbHelper = new DBHelper(this.context);
		db = dbHelper.getWritableDatabase();
		isOpen = true;
		return this;
	}
	
	/**
	 * Chiude la connessione
	 */
	public void close(){
		db.close();
		isOpen = false;
	}
	
	/**
	 * Inserisce una nuova entry dalla res
	 * @param res Reservation da inserire
	 * @return id Numero di riga del nuovo inserimento (-1 se c' errore)
	 */
	public long createReservation(Reservation res){		
		ContentValues content = new ContentValues();
		content.put(KEY_NAME, res.getName());
		content.put(KEY_ADDRESS, res.getAddress());
		content.put(KEY_PORT, res.getPort());
		content.put(KEY_PASSWORD, res.getPassword());
		content.put(KEY_WAITING, res.getWaiting());
		content.put(KEY_LASTUPDATE, res.getLastUpdate());
		content.put(KEY_STATUS, res.getStatus());
		content.put(KEY_MSG, res.getMsgFromServer());
		rowCount += 1;
		open();
		long i = db.insert(DATABASE_TABLE, null, content);
		close();
		return i;
	}
	
	/**
	 * Ritorna una reservation dal database
	 * @param id ID della reservation
	 * @return null se non esiste una Reservation con quell'id
	 */
	public Reservation fetchReservation(long id){
		open();
		Cursor cursor = db.query(DATABASE_TABLE, ALL_COLUMNS, KEY_ROWID + "='" + id + "'", null, null, null, null);
		Reservation res = null;
		if(cursor.moveToFirst()){ //se cursor  vuoto ritorna False
			res = new Reservation(cursor.getString(cursor.getColumnIndex(KEY_NAME)), 
									cursor.getString(cursor.getColumnIndex(KEY_ADDRESS)), 
									cursor.getInt(cursor.getColumnIndex(KEY_PORT)), 
									cursor.getString(cursor.getColumnIndex(KEY_PASSWORD)));
			res.setStatus(cursor.getInt(cursor.getColumnIndex(KEY_STATUS)));
			res.setWaiting(cursor.getLong(cursor.getColumnIndex(KEY_WAITING)));
			res.setLastUpdate(cursor.getLong(cursor.getColumnIndex(KEY_LASTUPDATE)));
			res.setMsgFromServer(cursor.getString(cursor.getColumnIndex(KEY_MSG)));
		}
		close();
		cursor.close();
		return res;
	}
	
	/**
	 * Ricava l'id di una Reservation
	 * @param res Reservation da cercare
	 * @return id della Reservation o -1 se Reservation non trovata
	 */
	public long getIdFromResevation(Reservation res){
		long id = -1;
		String WHERE = KEY_NAME + "='" + res.getName() + "' " 
				+ "AND " + KEY_ADDRESS + "='" + res.getAddress() + "' "
				+ "AND " +KEY_PORT + "=" + res.getPort() + " "
				+ "AND " +KEY_PASSWORD + "='" + res.getPassword() + "'";
		
		open();
		Cursor cursor = db.query(DATABASE_TABLE, new String[]{KEY_ROWID}, WHERE, null, null, null, null);
		if(cursor.moveToFirst())
			id = cursor.getLong(cursor.getColumnIndex(KEY_ROWID));
		cursor.close();
		close();
		return id;
	}
	
	/**
	 * Restituisce un Cursor con tutte le reservation presenti nel database
	 * @return Cursor a tutte le reservation
	 */
	public Cursor fetchAllReservations(){
		rowReady = 0;
		open();
		Cursor cursor = db.query(DATABASE_TABLE, LISTVIEW_COLUMNS, null, null, null, null, null);
		rowCount = cursor.getCount();
		while(cursor.moveToNext()){
			int status = cursor.getInt(cursor.getColumnIndex(KEY_STATUS));
			if(status == Reservation.READY || status == Reservation.CONFIRMED  )
				rowReady ++;
		}
		
		cursor.moveToFirst();
		close();
		return cursor;
	}
	
	/**
	 * Rimuove la entry indicata in id
	 * @param id della Reservation da rimuovere
	 * @return il risultato della rimozione
	 */
	public boolean removeReservation(long id){
		open();
		Cursor cursor = db.query(DATABASE_TABLE, new String[]{KEY_STATUS}, KEY_ROWID + "=" + id, null, null, null, null);
		cursor.moveToNext();
		int status = cursor.getInt(0); // 0, dato che  solo una colonna...
		cursor.close();
		if(status == Reservation.READY)
			rowReady --;
		rowCount --;		
		int i = db.delete(DATABASE_TABLE, KEY_ROWID + "=" + id, null);
		close();
		return i > 0;
		
	}
	
	/**
	 * Drop della tabella e creazione di una nuova ( pi performante di cancellare tutte le entry)
	 */
	public void removeAllReservation(){
		open();
		db.execSQL("drop table if exists " + DATABASE_TABLE); // Pi performante di cancellare una riga alla volta...
		dbHelper.onCreate(db);
		close();
		rowCount = 0;
		rowReady = 0;
	}
	
	public boolean updateReservation(long id, Reservation res){
		open();
		ContentValues args = new ContentValues();
        args.put(KEY_STATUS, res.getStatus());
        args.put(KEY_WAITING, res.getWaiting());
        args.put(KEY_LASTUPDATE, res.getLastUpdate());
        args.put(KEY_MSG, res.getMsgFromServer());
        String where = KEY_ROWID + "=" + id;
        if(res.getStatus() == Reservation.READY)
        	rowReady ++; 
        int i = db.update(DATABASE_TABLE, args, where, null);
        close();
        return i > 0;
	}
	
	public boolean isOpen(){
		return isOpen;
	}
	
	public boolean isEmpty(){
		return rowCount == 0;
	}
	
	public boolean isFullOfReady(){
		return rowCount - rowReady == 0;
	}
	
	
	
////////////////////////////////////////////////////////////////////////////////////////////////////////////////	
	/**
	 * Apre ed aggiorna il database
	 */
	private class DBHelper extends SQLiteOpenHelper{

		public DBHelper(Context context){
			super(context, DATABASE_NAME, null, DATABASE_VERSION);
		}

		@Override
		public void onCreate(SQLiteDatabase db) {
			db.execSQL(DATABASE_CREATE);
		}

		@Override
		public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
			db.execSQL("drop table if exists " + DATABASE_TABLE);
			onCreate(db);
			
		}
	}
}
