

import java.util.Vector;
import java.util.concurrent.TimeUnit;


public class QueueMonitor {

	private Vector<GenericCustomer> queue;
	private long totalWaiting;

	public QueueMonitor(){
		queue = new Vector<GenericCustomer>();
		totalWaiting = 0;
		
	}
	
	/**
	 * Inserisce un customer in coda
	 * @param customer da inserire in coda
	 * @return false se gi presente in coda
	 */
	public boolean insertInQueue(Customer customer){
		if(isPresent(customer))
			return false;
		System.out.println(customer.id +" si accoda\t\t\t" + getReadableTime(customer.serviceTime) + "\t" + queue.size() + "\t" + getReadableTime(totalWaiting) + "\t" + ((customer.isPatient)?"senza fretta":getReadableTime(customer.maxWaiting)));

		totalWaiting += customer.serviceTime;	
		queue.add(customer);
		return true;
	}
	
	/**
	 * Inserisce un customer KIQ in coda
	 * @param customer KIQ da inserire in coda
	 * @return false se gi presente in coda
	 */
	public boolean insertKIQInQueue(KeepInQCustomer customer){
		if(isPresent(customer))
			return false;
		System.out.println("KIQ:" + customer.id +" si accoda\t\t\t" + queue.size() + "\t" + getReadableTime(totalWaiting));
		((KeepInQCustomer)customer).waiting = totalWaiting;
		
		totalWaiting += customer.serviceTime;	
		queue.add(customer);
		return true;
	}

	/**
	 * Segnala che il customer sta lasciando la coda. Se ne avr una conferma solo quando 
	 * l'Operator prover a servire quel Customer, che di fatto non c' pi
	 * @param id del customer che sta lasciando la coda
	 * @return false se non presente in coda
	 */
	public boolean leaveQueue(String id) {
		int pos = positionOf(id);
		if(pos >= 0){
			queue.elementAt(pos).isStillInQueue = false;
			return true;
		}
		return false;	
	}
	
	/**
	 * Rimuove il customer KIQ dalla coda
	 * @param id del customer KIQ
	 * @return true se rimosso, false altrimenti (se non in coda)
	 */
	public boolean leaveQueueKIQ(String id){
		int pos = positionOf(id);
		if(pos >= 0){
			totalWaiting -= queue.elementAt(pos).serviceTime;
			queue.remove(pos);
			System.out.println("KIQ:" + id +" lascia la coda");
			return true;
		}
		System.out.println("KIQ:" + id +" non in coda");
		return false;
	}
	
	/**
	 * Conferma la prenotazione KIQ
	 * @param customer cliente KIQ che conferma la prenotazione
	 * @return true se confermata, false altrimenti (cio se non  presente in coda)
	 */
	public boolean confirmReservation(KeepInQCustomer customer){
		int pos = positionOf(customer);
		if(pos >= 0){
			((KeepInQCustomer)queue.elementAt(pos)).confirmed = true;
			System.out.println("KIQ:" + customer.id +" conferma la prenotazione");
			return true;
		}
		
		return false;
	}
	
	/**
	 * Estrae il prossimo in coda e lo restituisce.
	 * @return null se la coda  vuota
	 */
	public GenericCustomer getNextInQueue(){
		if(queue.size() > 0){
			GenericCustomer customer = queue.remove(0);
			return customer;
		}
		return null;
	}
	
	/**
	 * Restituisce il tempo di attesa di un KIQcustomer 
	 * @param id del KIQcustomer
	 * @return tempo di attesa, -1 se non presente
	 */
	public long getKIQCustomerWaiting(String id){
		int pos = positionOf(id);
		if(pos >= 0){
			return ((KeepInQCustomer)queue.elementAt(pos)).waiting;	
		}
		return -1;
	}

	/**
	 * Controlla se la coda  vuota
	 * @return true se vuota, false altrimenti
	 */
	public boolean isEmpty(){
		return queue.size() == 0;
	}

	
	/**
	 * Ritorna il numero di clienti in coda
	 * @return il numero di clienti in coda
	 */
	public int getCustomersInQueue(){
		return queue.size();
	}
	
	/**
	 * Controlla se un customer  gi presente
	 * @param customer da ricercare
	 * @return true se gi presente, false altrimenti
	 */
	private boolean isPresent(GenericCustomer customer) {
		for(int i = 0; i < queue.size(); i++)
			if(queue.elementAt(i).id.compareToIgnoreCase(customer.id) == 0)
				return true;
		return false;
	}

	/**
	 * Restituisce la posizione di un customer
	 * @param customer da ricercare
	 * @return la posizione nella coda, -1 se non presente
	 */
	private int positionOf(GenericCustomer customer) {
		for(int i = 0; i < queue.size(); i++)
			if(queue.elementAt(i).id.compareToIgnoreCase(customer.id) == 0)
				return i;
		return -1;
	}

	/**
	 * Restituisce la posizione di un customer dal suo id
	 * @param id da ricercare
	 * @return la posizione nella coda, -1 se non presente
	 */
	private int positionOf(String id) {
		for(int i = 0; i < queue.size(); i++){
			if(queue.elementAt(i).id.compareToIgnoreCase(id) == 0)
				return i;
		}
		return -1;
	}
	
	
	/**
	 * Restituisce un orario nel formato MM:SS partendo da un Long
	 * @param l Millisecondi da convertire
	 * @return 
	 */
	public String getReadableTime(long l) {
		// se cambia qui deve cambiare anche in QueueMonitor
		int m = (int) TimeUnit.MILLISECONDS.toMinutes(l);
		int s = (int) TimeUnit.MILLISECONDS.toSeconds(l - TimeUnit.MINUTES.toMillis(TimeUnit.MILLISECONDS.toMinutes(l)));
		return ((m<10)? "0"+m : m) + "m" + ((s<10)? "0"+s : s) + "s" ; 
	}

	/**
	 * Ritorna l'attesa totale della coda
	 * @return l'attesa totale della coda
	 */
	public long getWaitingTime() {
		return totalWaiting;
	}

	/**
	 * Simula l'inizio del servizio di un customer, quindi riduce l'attesa totale in coda 
	 * e anche quella dei KIQ customer
	 * @param customer
	 */
	public void startService(GenericCustomer customer) {
		totalWaiting -= customer.serviceTime;
		for(int i = 0; i < queue.size(); i++){
			if(queue.elementAt(i).isKIQ)
				//Aggiorno il tempo di attesa del customer KIQ
				((KeepInQCustomer)queue.elementAt(i)).waiting -= customer.serviceTime;
		}
	}
}