
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.concurrent.TimeUnit;

import com.ddignani.network.ClientRequest;
import com.ddignani.network.ServerResponse;

class ServerThread extends Thread {
	private QueueMonitor queueMonitor;
	private Operator operator;
	
	private Socket clientSocket = null;
	ClientRequest dataFromClient;
	ServerResponse dataToClient; 
	ObjectInputStream oIn;
	ObjectOutputStream oOut;

	public ServerThread(Socket socket, QueueMonitor queueMonitor, Operator operator) {
		super("ServerThread");
		clientSocket = socket;
		this.queueMonitor = queueMonitor;	
		this.operator = operator;
	} 

	public void run() {
		try {
			oOut = new ObjectOutputStream(clientSocket.getOutputStream());
			oIn  = new ObjectInputStream(clientSocket.getInputStream());
			try {
				dataFromClient = getFromClient(oIn);
				dataToClient = processInput(dataFromClient);
				
				switch(dataToClient.getStatus()){
					case ServerResponse.ENQUEUED:
					case ServerResponse.UPDATED:
					case ServerResponse.READY:
					case ServerResponse.ERROR:
						sendDataToClient(dataToClient, oOut);
						break;
						
					case ServerResponse.DEQUEUED:
						break;						
				}
				
			}catch (Exception e) {
				e.printStackTrace();
			}

			// Chiudi tutto
			oOut.close();
			oIn.close();
			clientSocket.close();
		} catch ( IOException e ) {
			e.printStackTrace();
		}
	} 



	private static ClientRequest getFromClient(ObjectInputStream oIn) throws IOException {
		ClientRequest  dataFromClient = null;         
		try {
			dataFromClient = (ClientRequest) oIn.readObject();
		} catch ( ClassNotFoundException e ) {
			System.out.println( "ClassNotFound" );
		}
		return dataFromClient;
	}


	private static void sendDataToClient(ServerResponse dataToClient, ObjectOutputStream  oOut) throws IOException {         
		oOut.writeObject(dataToClient);
		return;
	}
	
	
	private ServerResponse processInput(ClientRequest dataFromClient) {
		ServerResponse response = new ServerResponse();
		long w;
		
		switch (dataFromClient.getAction()) {
		case ClientRequest.NEW_RESERVATION:
			if(queueMonitor.isEmpty() && !operator.isBusy()){
				response.setStatus(ServerResponse.READY);
				response.setTime(0);
				response.setMoreInfo("There's no need to wait!");
			}else{			
				KeepInQCustomer customer = new KeepInQCustomer(queueMonitor, dataFromClient.getIdSender());
				boolean enqueued = queueMonitor.insertKIQInQueue(customer);
				w = queueMonitor.getKIQCustomerWaiting(dataFromClient.getIdSender());	
				if(enqueued){
					if(w <= TimeUnit.MINUTES.toMillis(3)){ // mancano meno di 3 minuti
						response.setStatus(ServerResponse.READY);
						response.setTime(w);
						response.setMoreInfo("It's almost your turn!");
					}else{
						response.setStatus(ServerResponse.ENQUEUED);
						response.setTime(queueMonitor.getWaitingTime());
						response.setMoreInfo("Estimated waiting...");
					}
				}else // gi presente in coda -> forse si sta cercando di confermare la prenotazione...
					if(w > TimeUnit.MINUTES.toMillis(3)){ // non ci si pu confermare se mancano pi di 3 minuti
					response.setStatus(ServerResponse.ERROR);
					response.setMoreInfo("It's too early to confirm this reservation");
					}else{
						queueMonitor.confirmReservation(customer);
						response.setStatus(ServerResponse.CONFIRMED);
						response.setMoreInfo("Reservation confirmed");
					}
			}
			break;
			
		case ClientRequest.DECLINE_RESERVATION:	
			queueMonitor.leaveQueueKIQ(dataFromClient.getIdSender());
			response.setStatus(ServerResponse.DEQUEUED);
			break;
		
		case ClientRequest.REFRESH_RESERVATION:
			w = queueMonitor.getKIQCustomerWaiting(dataFromClient.getIdSender());
			System.out.println("KIQ:" + dataFromClient.getIdSender() + " aggiorna stato\t\t\t" + queueMonitor.getReadableTime(w));
			if(w < 0){
				response.setStatus(ServerResponse.ERROR);
				response.setMoreInfo("Not present in this queue");
			}else if(w <= TimeUnit.MINUTES.toMillis(3)){ // mancano meno di 3 minuti
				response.setStatus(ServerResponse.READY);
				response.setTime(w);
				response.setMoreInfo("It's almost your turn!!");
			}else{	// manca ancora del tempo...
				response.setStatus(ServerResponse.UPDATED);
				response.setTime(w);
				response.setMoreInfo("Estimated waiting...");
			}
			break;
			
		default:
			response.setStatus(ServerResponse.ERROR);
			break;
		}
		return response;
	}
} 
